@@ -1,19 +1,34 @@
-// Copyright (C) 2000-2010 Heiko Bauke <heiko.bauke@mpi-hd.mpg.de>
-//  
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License in
-// version 2 as published by the Free Software Foundation.
-//  
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//  
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
-//  
+// Copyright (c) 2000-2010, Heiko Bauke
+// All rights reserved.
+// 
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions
+// are met:
+// 
+//   * Redistributions of source code must retain the above copyright
+//     notice, this list of conditions and the following disclaimer.  
+// 
+//   * Redistributions in binary form must reproduce the above
+//     copyright notice, this list of conditions and the following
+//     disclaimer in the documentation and/or other materials provided
+//     with the distribution.  
+// 
+//   * Neither the name of the copyright holder nor the names of its
+//     contributors may be used to endorse or promote products derived
+//     from this software without specific prior written permission.
+// 
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+// COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
+// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+// OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <trng/lcg64_shift.hpp>
 
@@ -45,6 +60,58 @@ namespace trng {
     return not (S1==S2);
   }
   
+  // compute floor(log_2(x))
+  unsigned int lcg64_shift::log2_floor(lcg64_shift::result_type x) {
+    unsigned int y(0);
+    while (x>0) {
+      x>>=1;
+      ++y;
+    };
+    --y;
+    return y;
+  }
+
+  // compute pow(x, n)
+  lcg64_shift::result_type lcg64_shift::pow(lcg64_shift::result_type x, lcg64_shift::result_type n) {
+    lcg64_shift::result_type result=1;
+    while (n>0) {
+      if ((n&1)>0)
+        result=result*x;
+      x=x*x;
+      n>>=1;
+    }
+    return result;
+  }
+
+  // compute prod(1+a^(2^i), i=0..l-1)
+  lcg64_shift::result_type lcg64_shift::g(unsigned int l, lcg64_shift::result_type a) {
+    lcg64_shift::result_type p=a, res=1;
+    for (unsigned i=0; i<l; ++i) {
+      res*=1+p;
+      p*=p;
+    }
+#if ULONG_LONG_MAX>18446744073709551615ull
+    return res & 0xfffffffffffffffful;
+#else
+    return res;
+#endif
+  }
+  
+  // compute sum(a^i, i=0..s-1)
+  lcg64_shift::result_type lcg64_shift::f(lcg64_shift::result_type s, lcg64_shift::result_type a) {
+    if (s==0)
+      return 0;
+    unsigned int l=log2_floor(s);
+    lcg64_shift::result_type p=a;
+    for (unsigned i=0; i<l; ++i)
+      p*=p;
+#if ULONG_LONG_MAX>18446744073709551615ull
+    return ( g(l, a)+p*f(s-(1ull<<l), a) ) & 0xfffffffffffffffful;
+#else
+    return g(l, a)+p*f(s-(1ull<<l), a);
+#endif
+  }
+
   const lcg64_shift::parameter_type
   lcg64_shift::Default=parameter_type(18145460002477866997ull, 1ull);
   const lcg64_shift::parameter_type 
@@ -84,7 +151,7 @@ namespace trng {
   }
 
   bool operator!=(const lcg64_shift &R1, const lcg64_shift &R2) {
-    return !(R1==R2);
+    return not (R1==R2);
   }
 
   // Parallel random number generator concept
@@ -93,28 +160,17 @@ namespace trng {
       throw std::invalid_argument("invalid argument for trng::lcg64_shift::split");
     if (s>1) {
       jump(n+1);
-      lcg64_shift::result_type t1(1ull), t2(0ull);
-      for (unsigned int i(0); i<s; ++i) {
-	t2+=t1;
-	t1*=P.a;
-      }
-      P.b*=t2;
-      P.a=t1;
+      P.b*=f(s, P.a);
+      P.a=pow(P.a, s);
       backward();
     }
   }
 
   void lcg64_shift::jump2(unsigned int s) {
-    lcg64_shift::result_type t1(P.a);
-    for (unsigned int i(0); i<s; ++i)
-      t1*=t1;
-    lcg64_shift::result_type t2(1ull), t3(P.a);
-    while (s>0l) {
-      t2*=(1ull+t3);
-      t3*=t3;
-      --s;
-    }
-    S.r=S.r*t1+t2*P.b;
+    S.r=S.r*pow(P.a, 1ull<<s)+f(1ull<<s, P.a)*P.b;
+#if ULONG_LONG_MAX>18446744073709551615ull
+    S.r&=0xfffffffffffffffful;
+#endif
   }
 
   void lcg64_shift::jump(unsigned long long s) {
