@@ -1,19 +1,34 @@
-// Copyright (C) 2000-2010 Heiko Bauke <heiko.bauke@mpi-hd.mpg.de>
-//  
-// This program is free software; you can redistribute it and/or
-// modify it under the terms of the GNU General Public License in
-// version 2 as published by the Free Software Foundation.
-//  
-// This program is distributed in the hope that it will be useful, but
-// WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// General Public License for more details.
-//  
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-// 02111-1307, USA.
-//  
+// Copyright (c) 2000-2010, Heiko Bauke
+// All rights reserved.
+// 
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions
+// are met:
+// 
+//   * Redistributions of source code must retain the above copyright
+//     notice, this list of conditions and the following disclaimer.  
+// 
+//   * Redistributions in binary form must reproduce the above
+//     copyright notice, this list of conditions and the following
+//     disclaimer in the documentation and/or other materials provided
+//     with the distribution.  
+// 
+//   * Neither the name of the copyright holder nor the names of its
+//     contributors may be used to endorse or promote products derived
+//     from this software without specific prior written permission.
+// 
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+// COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
+// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
+// OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #if !(defined TRNG_LCG64_SHIFT_HPP)
 
@@ -40,7 +55,14 @@ namespace trng {
     static const result_type min=0;
     static const result_type max=18446744073709551615ull;
   private:
-    static const bool use_mask=math::numeric_limits<result_type>::digits>64;
+    // compute floor(log_2(x))
+    static unsigned int log2_floor(result_type x);
+    // compute pow(x, n)
+    static result_type pow(result_type x, result_type n);
+    // compute prod(1+a^(2^i), i=0..l-1)
+    static result_type g(unsigned int l, result_type a);
+    // compute sum(a^i, i=0..s-1)
+    static result_type f(result_type s, result_type a);
   public:
     // Parameter and status classes
     class parameter_type;
@@ -224,19 +246,21 @@ namespace trng {
   // Inline and template methods
   
   inline void lcg64_shift::step() const {
-    if (use_mask)
-      S.r=(P.a*S.r+P.b) & max;
-    else
-      S.r=(P.a*S.r+P.b);
-    }
+#if ULONG_LONG_MAX>18446744073709551615ull
+    S.r=(P.a*S.r+P.b) & max;
+#else
+    S.r=(P.a*S.r+P.b);
+#endif
+  }
     
   inline lcg64_shift::result_type lcg64_shift::operator()() const {
     step();
     unsigned long long t=S.r;
     t^=(t>>17);
     t^=(t<<31);
-    if (use_mask)
+#if ULONG_LONG_MAX>18446744073709551615ull
       t&=max;
+#endif
     t^=(t>>8);
     return t;
   }
